/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2017, Christopher Friedt <chrisfriedt@gmail.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include <arpa/inet.h>
#include <libgen.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <unistd.h>

#include <cerrno>
#include <cstdlib>
#include <iostream>
#include <thread>
#include <string>

#include <boost/asio.hpp>

class ctx;

static void tx_thread_fn( ctx &ctx );
static void rx_thread_fn( ctx &ctx );

class ctx {

public:

	std::string host;
	uint16_t port;

	bool should_stop;

	boost::asio::ip::tcp::iostream fds;

	std::thread tx_thread;
	std::thread rx_thread;

	ctx()
	: ctx( "127.0.0.1", 1234 )
	{
	}

	ctx( const std::string & host, const uint16_t port )
	:
		host( host ),
		port( port ),
		should_stop( false ),
		fds( host, std::to_string( port ) ),
		tx_thread( tx_thread_fn, std::ref( *this ) ),
		rx_thread( rx_thread_fn, std::ref( *this ) )
	{
	}

	~ctx() {
		close();
	}

	void close() {
		should_stop = false;
		fds.close();
		tx_thread.join();
		rx_thread.join();
	}
};

/*
static void usage( const std::string & progname ) {
	std::cout
		<< "usage(): "
		<< progname
		<< " <ip>"
		<< " <port>"
		<< std::endl
		;
}
*/

static void tx_thread_fn( ctx &ctx ) {
	std::string line;
	for( ; ! ctx.should_stop ; ) {
		std::getline( std::cin, line );
		ctx.fds << line << std::flush;
	}
}

static void rx_thread_fn( ctx &ctx ) {
	std::string line;
	for( ; ! ctx.should_stop ; ) {
		std::getline( ctx.fds, line );
		if ( line.empty() ) {
			ctx.should_stop = true;
			continue;
		}
		std::cout << "<= '" << line << "'" << std::endl;
	}
}

int main( int argc, char *argv[] ) {

	ctx ctx;

	for( ; ! ctx.should_stop; ) {
		sleep( 1 );
	}

	return 0;
}
