namespace java org.gdb.rsp
namespace py org.gdb.rsp
namespace cpp org.gdb.rsp

enum GdbRspError {
	GDB_NO_ERROR,
	GENERIC_ERROR,
	NOT_FOUND_ERROR,
	TLS_NO_LIBRARY_SUPPORT_ERROR,
	TLS_LOAD_MODULE_NOT_FOUND_ERROR,
	TLS_NOT_ALLOCATED_YET_ERROR,
	TLS_GENERIC_ERROR,
	XML_PARSE_ERROR,
	MEMORY_ERROR,
	NOT_AVAILABLE_ERROR,
	OPTIMIZED_OUT_ERROR,
	NO_ENTRY_VALUE_ERROR,
	TARGET_CLOSE_ERROR,
	UNDEFINED_COMMAND_ERROR,
	NOT_SUPPORTED_ERROR,
	MAX_COMPLETIONS_REACHED_ERROR,
}

exception GdbRspException {
	1: GdbRspError err_no,
	2: string msg,
}
// error values

// TODO: @CF: 20170905: use a constant map for all of the stuff that was written by hand in C++
// See http://thrift-tutorial.readthedocs.io/en/latest/usage-example.html
typedef map<string,string> GdbFeature;
typedef map<string,string> StubFeature;

struct QuerySupportedResponse {
	1: StubFeature feat,
}

typedef i8 SetThreadOp
const SetThreadOp SET_THREAD_OP_STEP_AND_CONTINUE = 99 // 'c'
const SetThreadOp SET_THREAD_OP_OTHER = 103 // 'g'

typedef i32 ThreadId
const ThreadId THREAD_ID_ALL = -1
const ThreadId THREAD_ID_ANY = 0

struct QueryXferFeaturesReadResponse {
	1: string data,
	2: bool more
}

struct QueryTraceStatusResponseField {
	1: string field,
	2: string text,
	3: i32 num
}
struct QueryTraceStatusResponse {
	1: bool running,
	2: list<QueryTraceStatusResponseField> fields
}

typedef string StopReplyType
const StopReplyType STOP_REPLY_OK = "OK"
const StopReplyType STOP_REPLY_SIG = "S"
const StopReplyType STOP_REPLY_SIG_INFO = "T"
const StopReplyType STOP_REPLY_PROC_EXIT = "W"
const StopReplyType STOP_REPLY_PROC_TERM = "X"
const StopReplyType STOP_REPLY_THREAD_EXIT = "w"
const StopReplyType STOP_REPLY_NO_THREADS = "N"
const StopReplyType STOP_REPLY_DEBUG_CONSOLE = "O"
const StopReplyType STOP_REPLY_HOST_SYSCALL = "F"

struct StopReplyPacket {
	1: StopReplyType type,
	2: string hexstr,
	3: i32 intval,
	4: map<string,string> data
}

service GdbRsp {
	void backwardContinue() throws( 1: GdbRspException e ), // "bc"
	void backwardSingleStep() throws( 1: GdbRspException e ), // "bs"
	oneway void sigint(), // "^C" / 0x03
	void consoleOutput() throws( 1: GdbRspException e ), // "O"
	void continue_() throws( 1: GdbRspException e ), // "c"
	void continueWithSignal() throws( 1: GdbRspException e ), // "C"
	void createTracePoint() throws( 1: GdbRspException e ), // "QTDP:"
	void createTraceStateVariable() throws( 1: GdbRspException e ), // "QTDV:"
	void cycleStep() throws( 1: GdbRspException e ), // "i"
	void cycleStepAfterSignal() throws( 1: GdbRspException e ), // "I"
	void defineTracePointAction() throws( 1: GdbRspException e ), // "QTDP:-"
	void detach() throws( 1: GdbRspException e ), // "D"
	void enableExtendedMode() throws( 1: GdbRspException e ), // "!"
	void expeditedTargetStatusResponse() throws( 1: GdbRspException e ), // "T"
	void insertBreakpoint() throws( 1: GdbRspException e ), // "Z"
	void kill() throws( 1: GdbRspException e ), // "k"
	void passSignals() throws( 1: GdbRspException e ), // "QPassSignals"
	void programSignals() throws( 1: GdbRspException e ), // "QProgramSignals"
	void queryAttached() throws( 1: GdbRspException e ), // "qAttached"
	void queryCRC() throws( 1: GdbRspException e ), // "qCRC"
	void queryCurrentThreadID() throws( 1: GdbRspException e ), // "qC"
	void queryFirstThreadInfo() throws( 1: GdbRspException e ), // "qfThreadInfo"
	void queryGetTLSAddress() throws( 1: GdbRspException e ), // "qGetTLSAddr"
	void queryGetThreadInformationBlockAddress() throws( 1: GdbRspException e ), // "qGetTIBAddr"
	void queryLThreadInfo() throws( 1: GdbRspException e ), // "qL"
	void queryPThreadInfo() throws( 1: GdbRspException e ), // "qP"
	void queryRelocInsn() throws( 1: GdbRspException e ), // "qRelocInsn"
	void queryRunCommand() throws( 1: GdbRspException e ), // "qRcmd"
	void querySearchMemory() throws( 1: GdbRspException e ), // "qSearch"
	void querySectionOffsets() throws( 1: GdbRspException e ), // "qOffsets"
	void querySubsequentThreadInfo() throws( 1: GdbRspException e ), // "qsThreadInfo"
	QuerySupportedResponse querySupported( 1: GdbFeature arg1 ) throws( 1: GdbRspException e ), // "qSupported"
	void querySymbol() throws( 1: GdbRspException e ), // "qSymbol"
	void queryThreadExtraInfo() throws( 1: GdbRspException e ), // "qThreadExtraInfo"
	void queryTraceFirstPoint() throws( 1: GdbRspException e ), // "qTfP:"
	void queryTraceFirstStaticMarker() throws( 1: GdbRspException e ), // "qTfSTM"
	void queryTraceFirstVariable() throws( 1: GdbRspException e ), // "qTfV"
	void queryTraceMinimumFastTracepointInstructionLength() throws( 1: GdbRspException e ), // "qTMinFTPILen"
	void queryTracePoint() throws( 1: GdbRspException e ), // "qTP:"
	void queryTraceStateVariable() throws( 1: GdbRspException e ), // "qTV:"
	void queryTraceStaticMarkerAt() throws( 1: GdbRspException e ), // "qTSTMat:"
	QueryTraceStatusResponse queryTraceStatus() throws( 1: GdbRspException e ), // "qTStatus"
	void queryTraceSubsequentPoint() throws( 1: GdbRspException e ), // "qTsP:"
	void queryTraceSubsequentStaticMarker() throws( 1: GdbRspException e ), // "qTsSTM"
	void queryTraceSubsequentVariable() throws( 1: GdbRspException e ), // "qTsV"
	void queryTracebuffer() throws( 1: GdbRspException e ), // "qTBuffer:"
	QueryXferFeaturesReadResponse queryXferFeaturesRead( 1: string arg1, 2: i32 arg2, 3: i32 arg3 ) throws( 1: GdbRspException e ), // "qXfer:features:read"
	void readMemory() throws( 1: GdbRspException e ), // "m"
	void readRegisterN() throws( 1: GdbRspException e ), // "p"
	void readRegisters() throws( 1: GdbRspException e ), // "g"
	void removeBreakpoint() throws( 1: GdbRspException e ), // "z"
	void reportMThreadInfo() throws( 1: GdbRspException e ), // "qM"
	StopReplyPacket reportTargetStatus() throws( 1: GdbRspException e ), // "?"
	void restart() throws( 1: GdbRspException e ), // "R"
	void saveTrace() throws( 1: GdbRspException e ), // "QTSave:"
	void searchMemory() throws( 1: GdbRspException e ), // "t"
	void selectTracepointFrame() throws( 1: GdbRspException e ), // "QTFrame:"
	void setAgent() throws( 1: GdbRspException e ), // "QAgent"
	void setAllow() throws( 1: GdbRspException e ), // "QAllow"
	void setArguments() throws( 1: GdbRspException e ), // "A"
	void setBaud() throws( 1: GdbRspException e ), // "b"
	void setBreakpoint() throws( 1: GdbRspException e ), // "B"
	void setDisableRandomization() throws( 1: GdbRspException e ), // "QDisableRandomization"
	void setNonStop() throws( 1: GdbRspException e ), // "QNonStop"
	void setThread( 1: SetThreadOp arg1, 2: ThreadId arg2 ) throws( 1: GdbRspException e ), // "H"
	void setTraceBufferCircular() throws( 1: GdbRspException e ), // "QTBuffer:circular:"
	void setTraceBufferSize() throws( 1: GdbRspException e ), // "QTBuffer:size:"
	void setTraceNotes() throws( 1: GdbRspException e ), // "QTNotes:"
	void specifyTracepointSourceString() throws( 1: GdbRspException e ), // "QTDPsrc:"
	void startNoAckMode() throws( 1: GdbRspException e ), // "QStartNoAckMode"
	void step() throws( 1: GdbRspException e ), // "s"
	void targetStatusResponse() throws( 1: GdbRspException e ), // "S"
	void terminateApplication() throws( 1: GdbRspException e ), // "k"
	void toggleDebug() throws( 1: GdbRspException e ), // "d"
	void traceDisconnected() throws( 1: GdbRspException e ), // "QTDisconnected"
	void traceMarkReadOnly() throws( 1: GdbRspException e ), // "QTro"
	void traceStart() throws( 1: GdbRspException e ), // "QTStart"
	void traceStop() throws( 1: GdbRspException e ), // "QTStop"
	void tracepointDisable() throws( 1: GdbRspException e ), // "QTDisable"
	void tracepointEnable() throws( 1: GdbRspException e ), // "QTEnable"
	void tracepointsInit() throws( 1: GdbRspException e ), // "QTinit"
	void vAttach() throws( 1: GdbRspException e ), // "vAttach"
	void vCont() throws( 1: GdbRspException e ), // "vCont"
	void vFile() throws( 1: GdbRspException e ), // "vFile"
	void vFlashDone() throws( 1: GdbRspException e ), // "vFlashDone"
	void vFlashErase() throws( 1: GdbRspException e ), // "vFlashErase"
	void vFlashWrite() throws( 1: GdbRspException e ), // "vFlashWrite"
	void vKill() throws( 1: GdbRspException e ), // "vKill"
	void vRun() throws( 1: GdbRspException e ), // "vRun"
	void vStopped() throws( 1: GdbRspException e ), // "vStopped"
	void writeMemory() throws( 1: GdbRspException e ), // "M"
	void writeMemoryBinary() throws( 1: GdbRspException e ), // "X"
	void writeRegisterN() throws( 1: GdbRspException e ), // "P"
	void writeRegisters() throws( 1: GdbRspException e ), // "G"
	oneway void reset(), // "r"
}