# The MIT License (MIT)
#
# Copyright (c) 2017, Christopher Friedt <chrisfriedt@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

#!/bin/sh

COPYRIGHT="$(head -n 1 COPYING)"
prepend() {
    local f="${1}"
    local sep="${2}"
    local pre="${3}"
    local post="${4}"
    
    head -n 5 "${f}" | grep "${COPYRIGHT}" &> /dev/null && echo "'${f}' already has copyright" && return
    head -n 5 "${f}" | grep -i 'Copyright' &> /dev/null && echo "WARNING: Conflicting copyright in '${f}': $(head -n 5 "${f}" | grep 'Copyright')" && return
    
    echo "processing file '${f}'"
    
    local tmpfile="$(mktemp -p "" $(basename ${f}).XXXXXX)"

    [[ "" != "${pre}" ]] && echo "${pre}" >> "${tmpfile}"
    while read line; do
        echo "${sep}${line}" | sed -e 's/[\ \t]\+$//g' >> "${tmpfile}"
    done < COPYING
    [[ "" != "${post}" ]] && echo "${post}" >> "${tmpfile}"
    
    echo "" >> "${tmpfile}"
    
    cat "${f}" >> "${tmpfile}"
    cp "${tmpfile}" "${f}"
    
    rm -f "${tmpfile}"
}

prepend_c() {
    
    local f="${1}"
    
    local sep=" * "
    local pre="/*"
    local post=" */"
    
    prepend "${f}" "${sep}" "${pre}" "${post}"
}

prepend_shebang() {
    
    local f="${1}"
    
    local sep="# "
    local pre=""
    local post=""
    
    prepend "${f}" "${sep}" "${pre}" "${post}"
}

sh autogen.sh clean || ( echo "failed to run 'sh autogen.sh clean'"; exit 1 )

echo "copyright: '${COPYRIGHT}'"

# Prepend license to all c-syntax files
for ext in c h cpp cc hpp js y l ypp lpp; do
    for f in $(find * -name "*.${ext}"); do
        prepend_c "${f}"
    done
done

# Prepend license to all sh-syntax files
for ext in sh in ac am py; do
    for f in $(find * -name "*.${ext}"); do
        if [ "$(basename ${0})" = "${f}" ]; then
            continue
        fi
        prepend_shebang "${f}"
    done
done