/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2017, Christopher Friedt <chrisfriedt@gmail.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#ifndef SRC_EXCEPTION_HPP_
#define SRC_EXCEPTION_HPP_

#include "gdbrsp_constants.h"

namespace org {
namespace gdb {
namespace rsp {
namespace exception {

static inline org::gdb::rsp::GdbRspException make( const org::gdb::rsp::GdbRspError::type err_no, const std::string & msg ) {
	GdbRspException ex;
	ex.err_no = err_no;
	ex.msg = msg;
	return ex;
}

class GdbRspUnknownPacket : public org::gdb::rsp::GdbRspException {
public:
	GdbRspUnknownPacket()
	: org::gdb::rsp::GdbRspException()
	{
		err_no = org::gdb::rsp::GdbRspError::NOT_SUPPORTED_ERROR;
	}

	GdbRspUnknownPacket( const std::string & _msg )
	: GdbRspUnknownPacket()
	{
		msg = _msg;
	}
};

} /* namespace exception */
} /* namespace rsp */
} /* namespace gdb */
} /* namespace org */

#endif /* SRC_EXCEPTION_HPP_ */
