/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2017, Christopher Friedt <chrisfriedt@gmail.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include <feature.hpp>

namespace org {
namespace gdb {
namespace rsp {

const std::vector<std::string> gdb_feature_s {
	"exec-events",
	"fork-events",
	"hwbreak",
	"multiprocess",
	"qRelocInsn",
	"read",
	"swbreak",
	"vContSupported",
	"vfork-events",
	"xmlRegisters",
};

const std::vector<std::string> stub_feature_s {
	"BreakpointCommands",
	"ConditionalBreakpoints",
	"ConditionalTracepoints",
	"EnableDisableTracepoints",
	"PacketSize",
	"QAgent",
	"QAllow",
	"QCatchSyscalls",
	"QDisableRandomization",
	"QNonStop",
	"QPassSignals",
	"QStartNoAckMode",
	"QTBuffer:size",
	"QThreadEvents",
	"Qbtrace-conf:bts:size",
	"Qbtrace-conf:pt:size",
	"Qbtrace:bts",
	"Qbtrace:off",
	"Qbtrace:pt",
	"ReverseContinue",
	"ReverseStep",
	"TracepointSource",
	"augmented-libraries-svr4-read",
	"exec-events",
	"fork-events",
	"hwbreak",
	"multiprocess",
	"no-resumed",
	"qXfer:auxv:read",
	"qXfer:btrace-conf:read",
	"qXfer:btrace:read",
	"qXfer:exec-file:read",
	"qXfer:fdpic:read",
	"qXfer:features:read",
	"qXfer:libraries-svr4:read",
	"qXfer:libraries:read",
	"qXfer:memory-map:read",
	"qXfer:sdata:read",
	"qXfer:siginfo:read",
	"qXfer:siginfo:write",
	"qXfer:spu:read",
	"qXfer:spu:write",
	"qXfer:threads:read",
	"qXfer:traceframe-info:read",
	"qXfer:uib:read",
	"swbreak",
	"tracenz",
	"vfork-events",
};

const std::map<std::string,stub_properties> default_stub_properties {
	{ "BreakpointCommands", stub_properties( false, "-", false ) },
	{ "ConditionalBreakpoints", stub_properties( false, "-", false ) },
	{ "ConditionalTracepoints", stub_properties( false, "-", false ) },
	{ "EnableDisableTracepoints", stub_properties( false, "-", false ) },
	{ "PacketSize", stub_properties( true, "-", false ) },
	{ "QAgent", stub_properties( false, "-", false ) },
	{ "QAllow", stub_properties( false, "-", false ) },
	{ "QCatchSyscalls", stub_properties( false, "-", true ) },
	{ "QDisableRandomization", stub_properties( false, "-", false ) },
	{ "QNonStop", stub_properties( false, "-", true ) },
	{ "QPassSignals", stub_properties( false, "-", true ) },
	{ "QStartNoAckMode", stub_properties( false, "-", true ) },
	{ "QTBuffer:size", stub_properties( false, "-", false ) },
	{ "QThreadEvents", stub_properties( false, "-", false ) },
	{ "Qbtrace-conf:bts:size", stub_properties( true, "-", true ) },
	{ "Qbtrace-conf:pt:size", stub_properties( true, "-", true ) },
	{ "Qbtrace:bts", stub_properties( true, "-", true ) },
	{ "Qbtrace:off", stub_properties( true, "-", true ) },
	{ "Qbtrace:pt", stub_properties( true, "-", true ) },
	{ "ReverseContinue", stub_properties( false, "-", false ) },
	{ "ReverseStep", stub_properties( false, "-", false ) },
	{ "TracepointSource", stub_properties( false, "-", false ) },
	{ "augmented-libraries-svr4-read", stub_properties( false, "-", false ) },
	{ "exec-events", stub_properties( false, "-", false ) },
	{ "fork-events", stub_properties( false, "-", false ) },
	{ "hwbreak", stub_properties( false, "-", false ) },
	{ "multiprocess", stub_properties( false, "-", false ) },
	{ "no-resumed", stub_properties( false, "-", false ) },
	{ "qXfer:auxv:read", stub_properties( false, "-", true ) },
	{ "qXfer:btrace-conf:read", stub_properties( false, "-", true ) },
	{ "qXfer:btrace:read", stub_properties( false, "-", true ) },
	{ "qXfer:exec-file:read", stub_properties( false, "-", true ) },
	{ "qXfer:fdpic:read", stub_properties( false, "-", true ) },
	{ "qXfer:features:read", stub_properties( false, "-", true ) },
	{ "qXfer:libraries-svr4:read", stub_properties( false, "-", true ) },
	{ "qXfer:libraries:read", stub_properties( false, "-", true ) },
	{ "qXfer:memory-map:read", stub_properties( false, "-", true ) },
	{ "qXfer:sdata:read", stub_properties( false, "-", true ) },
	{ "qXfer:siginfo:read", stub_properties( false, "-", true ) },
	{ "qXfer:siginfo:write", stub_properties( false, "-", true ) },
	{ "qXfer:spu:read", stub_properties( false, "-", true ) },
	{ "qXfer:spu:write", stub_properties( false, "-", true ) },
	{ "qXfer:threads:read", stub_properties( false, "-", true ) },
	{ "qXfer:traceframe-info:read", stub_properties( false, "-", true ) },
	{ "qXfer:uib:read", stub_properties( false, "-", true ) },
	{ "swbreak", stub_properties( false, "-", false ) },
	{ "tracenz", stub_properties( false, "-", false ) },
	{ "vfork-events", stub_properties( false, "-", false ) },
};

} /* namespace rsp */
} /* namespace gdb */
} /* namespace org */
