/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2017, Christopher Friedt <chrisfriedt@gmail.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#ifndef SRC_FEATURE_HPP_
#define SRC_FEATURE_HPP_

#include <map>
#include <string>
#include <vector>

namespace org {
namespace gdb {
namespace rsp {

extern const std::vector<std::string> gdb_feature_s;
extern const std::vector<std::string> stub_feature_s;

struct stub_properties {

	bool value_required;
	std::string value;
	bool probe_allowed;
	bool set;

	stub_properties()
	:
		stub_properties( false, "", false )
	{
	}

	stub_properties( const bool value_required, const std::string & value, const bool probe_allowed, const bool set = false )
	:
		value_required( value_required ),
		value( value ),
		probe_allowed( probe_allowed ),
		set( set )
	{
	}
};

extern const std::map<std::string,stub_properties> default_stub_properties;

} /* namespace rsp */
} /* namespace gdb */
} /* namespace org */

#endif /* SRC_FEATURE_HPP_ */
