/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2017, Christopher Friedt <chrisfriedt@gmail.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#ifndef SRC_PACKET_HPP_
#define SRC_PACKET_HPP_

#include <map>
#include <string>

namespace org {
namespace gdb {
namespace rsp {

extern const std::string framing;

typedef enum {
	ACK, // "+"
	BackwardContinue, // "bc"
	BackwardSingleStep, // "bs"
	CTRLC, // "^C"
	ConsoleOutput, // "O"
	Continue, // "c"
	ContinueWithSignal, // "C"
	CreateTracePoint, // "QTDP:"
	CreateTraceStateVariable, // "QTDV:"
	CycleStep, // "i"
	CycleStepAfterSignal, // "I"
	DefineTracePointAction, // "QTDP:-"
	Detach, // "D"
	EnableExtendedMode, // "!"
	ExpeditedTargetStatusResponse, // "T"
	InsertBreakpoint, // "Z"
	Kill, // "k"
	NACK, // "-"
	OK, // "OK"
	PassSignals, // "QPassSignals"
	ProgramSignals, // "QProgramSignals"
	QueryAttached, // "qAttached"
	QueryCRC, // "qCRC"
	QueryCurrentThreadID, // "qC"
	QueryFirstThreadInfo, // "qfThreadInfo"
	QueryGetTLSAddress, // "qGetTLSAddr"
	QueryGetThreadInformationBlockAddress, // "qGetTIBAddr"
	QueryLThreadInfo, // "qL"
	QueryPThreadInfo, // "qP"
	QueryRelocInsn, // "qRelocInsn"
	QueryRunCommand, // "qRcmd"
	QuerySearchMemory, // "qSearch"
	QuerySectionOffsets, // "qOffsets"
	QuerySubsequentThreadInfo, // "qsThreadInfo"
	QuerySupported, // "qSupported"
	QuerySymbol, // "qSymbol"
	QueryThreadExtraInfo, // "qThreadExtraInfo"
	QueryTraceFirstPoint, // "qTfP:"
	QueryTraceFirstStaticMarker, // "qTfSTM"
	QueryTraceFirstVariable, // "qTfV"
	QueryTraceMinimumFastTracepointInstructionLength, // "qTMinFTPILen"
	QueryTracePoint, // "qTP:"
	QueryTraceStateVariable, // "qTV:"
	QueryTraceStaticMarkerAt, // "qTSTMat:"
	QueryTraceStatus, // "qTStatus"
	QueryTraceSubsequentPoint, // "qTsP:"
	QueryTraceSubsequentStaticMarker, // "qTsSTM"
	QueryTraceSubsequentVariable, // "qTsV"
	QueryTracebuffer, // "qTBuffer:"
	QueryXferFeaturesRead, // "qXfer:features:read"
	ReadMemory, // "m"
	ReadRegisterN, // "p"
	ReadRegisters, // "g"
	RemoveBreakpoint, // "z"
	ReportMThreadInfo, // "qM"
	ReportTargetStatus, // "?"
	Restart, // "R"
	SaveTrace, // "QTSave:"
	SearchMemory, // "t"
	SelectTracepointFrame, // "QTFrame:"
	SetAgent, // "QAgent"
	SetAllow, // "QAllow"
	SetArguments, // "A"
	SetBaud, // "b"
	SetBreakpoint, // "B"
	SetDisableRandomization, // "QDisableRandomization"
	SetNonStop, // "QNonStop"
	SetThread, // "H"
	SetTraceBufferCircular, // "QTBuffer:circular:"
	SetTraceBufferSize, // "QTBuffer:size:"
	SetTraceNotes, // "QTNotes:"
	SpecifyTracepointSourceString, // "QTDPsrc:"
	StartNoAckMode, // "QStartNoAckMode"
	Step, // "s"
	TargetStatusResponse, // "S"
	TerminateApplication, // "k"
	ToggleDebug, // "d"
	TraceDisconnected, // "QTDisconnected"
	TraceMarkReadOnly, // "QTro"
	TraceStart, // "QTStart"
	TraceStop, // "QTStop"
	TracepointDisable, // "QTDisable"
	TracepointEnable, // "QTEnable"
	TracepointsInit, // "QTinit"
	VAttach, // "vAttach"
	VCont, // "vCont"
	VFile, // "vFile"
	VFlashDone, // "vFlashDone"
	VFlashErase, // "vFlashErase"
	VFlashWrite, // "vFlashWrite"
	VKill, // "vKill"
	VRun, // "vRun"
	VStopped, // "vStopped"
	WriteMemory, // "M"
	WriteMemoryBinary, // "X"
	WriteRegisterN, // "P"
	WriteRegisters, // "G"
	reset, // "r"
} packet_t;

extern std::map<packet_t,std::string> packet_s;


} /* namespace rsp */
} /* namespace gdb */
} /* namespace org */

#endif /* SRC_PACKET_HPP_ */
