/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2017, Christopher Friedt <chrisfriedt@gmail.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include <client.hpp>
#include <sstream>
#include <string>

#include <gtest/gtest.h>

#include "checksum.hpp"

namespace org {
namespace gdb {
namespace rsp {

class checksum_fixture
:
	public ::testing::Test
{

public:

	std::stringstream in;

	checksum_fixture()
	{
	}

	virtual ~checksum_fixture()
	{
	}

	uint8_t expected_checksum() {
		std::string s = in.str();
		s = s.substr( s.length() - 2, s.length() );
		return std::stoi( s, nullptr, 16 );
	}

	std::string expected_payload() {
		std::string s = in.str();
		s = s.substr( 1, s.length() - 4 );
		return s;
	}

protected:
	void SetUp();
	void TearDown();

	static void SetUpTestCase();
	static void TearDownTestCase();
};

void checksum_fixture::SetUpTestCase() {
}

void checksum_fixture::TearDownTestCase() {
}

void checksum_fixture::SetUp() {
}

void checksum_fixture::TearDown() {
}

TEST_F( checksum_fixture, checksum ) {
	uint8_t expected_uint8_t;
	uint8_t actual_uint8_t;

	in.str( "$PacketSize=1000;qXfer:features:read+#cc" );

	expected_uint8_t = expected_checksum();
	actual_uint8_t = org::gdb::rsp::checksum::calculate( expected_payload() );
	EXPECT_EQ( expected_uint8_t, actual_uint8_t );

	expected_uint8_t = 0xcc;
	EXPECT_EQ( expected_uint8_t, actual_uint8_t );
}

TEST_F( checksum_fixture, wrong_checksum ) {
	uint8_t expected_uint8_t;
	uint8_t actual_uint8_t;

	in.str( "$PacketSize=1000;qXfer:features:read+#cc" );
	expected_uint8_t = 0xff;
	actual_uint8_t = org::gdb::rsp::checksum::calculate( expected_payload() );
	EXPECT_NE( expected_uint8_t, actual_uint8_t );
}

} /* namespace rsp */
} /* namespace gdb */
} /* namespace org */
