/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2017, Christopher Friedt <chrisfriedt@gmail.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include <string>

#include <gtest/gtest.h>

#include "../src/hexstr.hpp"
#include "../src/vecstr.hpp"

namespace org {
namespace gdb {
namespace rsp {

TEST( HexStrTest, decode_simply ) {

	std::string expected_string;
	std::string actual_string;

	std::string input;

	input = "48656c6c6f2c20776f726c64210a";
	expected_string = "Hello, world!\n";

	actual_string = hexstr::decode( input );

	EXPECT_EQ( expected_string, actual_string ) << "expected: " << expected_string << " actual: " << actual_string << std::endl;
}

TEST( HexStrTest, encode_simply ) {

	std::string expected_string;
	std::string actual_string;

	std::string input;

	input = "Hello, world!\n";
	expected_string = "48656c6c6f2c20776f726c64210a";

	actual_string = hexstr::encode( input );

	EXPECT_EQ( expected_string, actual_string ) << "expected: " << expected_string << " actual: " << actual_string << std::endl;
}

TEST( HexStrTest, get_bytes ) {

	std::vector<uint8_t> expected_vector_uint8;
	std::vector<uint8_t> actual_vector_uint8;

	std::string input;

	input = "48656c6c6f2c20776f726c64210a";
	expected_vector_uint8 = std::vector<uint8_t>{ 0x48, 0x65, 0x6c, 0x6c, 0x6f, 0x2c, 0x20, 0x77, 0x6f, 0x72, 0x6c, 0x64, 0x21, 0x0a };

	actual_vector_uint8 = hexstr::get_bytes( input );

	EXPECT_EQ( expected_vector_uint8, actual_vector_uint8 )
		<< "expected: " << vec2str( expected_vector_uint8 )
		<< " actual: " << vec2str( actual_vector_uint8 )
		<< std::endl;
}

TEST( HexStrTest, get_byte_many ) {

	uint8_t expected_uint8;
	uint8_t actual_uint8;

	std::string input;

	input = "48656c6c6f2c20776f726c64210a";
	expected_uint8 = 0x48;

	actual_uint8 = hexstr::get_byte( input );

	EXPECT_EQ( expected_uint8, actual_uint8 )
		<< "expected: " << expected_uint8
		<< " actual: " << actual_uint8
		<< std::endl;
}

TEST( HexStrTest, get_byte_none ) {

	uint8_t expected_uint8;
	uint8_t actual_uint8;

	std::string input;

	input = "";
	expected_uint8 = -1;

	actual_uint8 = hexstr::get_byte( input );

	EXPECT_EQ( expected_uint8, actual_uint8 )
		<< "expected: " << expected_uint8
		<< " actual: " << actual_uint8
		<< std::endl;
}

} /* namespace rsp */
} /* namespace gdb */
} /* namespace org */
