/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2017, Christopher Friedt <chrisfriedt@gmail.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include <client.hpp>
#include <iostream>
#include <sstream>
#include <string>

#include <gtest/gtest.h>

#include "checksum.hpp"
#include "null_buffer.hpp"

namespace org {
namespace gdb {
namespace rsp {

class test_query_supported_fixture
:
	public ::testing::Test
{

public:

	std::ostream os;
	std::stringstream in;

	GdbFeature gdb_feature;

	test_query_supported_fixture()
	:
		os( & null_buffer )
	{
		in.unsetf( std::ios_base::skipws );
	}

	virtual ~test_query_supported_fixture()
	{
	}

protected:
	void SetUp();
	void TearDown();

	static void SetUpTestCase();
	static void TearDownTestCase();
};

void test_query_supported_fixture::SetUpTestCase() {
}

void test_query_supported_fixture::TearDownTestCase() {
}

void test_query_supported_fixture::SetUp() {
}

void test_query_supported_fixture::TearDown() {
}

TEST_F( test_query_supported_fixture, TestValidResponse ) {

	size_t expected_size_t;
	size_t actual_size_t;

	QuerySupportedResponse expected_qrs;
	QuerySupportedResponse actual_qrs;

	in.str( "+$PacketSize=1000;qXfer:features:read+#cc" );

	client client( in, os );

	gdb_feature = GdbFeature({
		{ "hwbreak", "+", },
		{ "swbreak", "+", },
		{ "qRelocInsn", "+", },
	});

	expected_qrs.feat = StubFeature({
		{ "PacketSize", "1000", },
		{ "qXfer:features:read", "+", },
	});

	client.querySupported( actual_qrs, gdb_feature );

	expected_size_t = expected_qrs.feat.size();
	actual_size_t = actual_qrs.feat.size();
	EXPECT_EQ( expected_size_t, actual_size_t );

	EXPECT_TRUE(
		true
		&& expected_qrs.feat.size() == actual_qrs.feat.size()
		&& std::equal(
			expected_qrs.feat.begin(),
			expected_qrs.feat.end(),
			actual_qrs.feat.begin()
		)
	);
}

} /* namespace rsp */
} /* namespace gdb */
} /* namespace org */
