/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2017, Christopher Friedt <chrisfriedt@gmail.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include <client.hpp>
#include <iostream>
#include <sstream>
#include <string>

#include <gtest/gtest.h>

#include "client.hpp"
#include "null_buffer.hpp"

namespace org {
namespace gdb {
namespace rsp {

class test_set_thread_fixture
:
	public ::testing::Test
{

public:

	std::ostream os;
	std::stringstream in;

	GdbFeature gdb_feature;

	test_set_thread_fixture()
	:
		os( & null_buffer )
	{
		in.unsetf( std::ios_base::skipws );
	}

	virtual ~test_set_thread_fixture()
	{
	}

protected:
	void SetUp();
	void TearDown();

	static void SetUpTestCase();
	static void TearDownTestCase();
};

void test_set_thread_fixture::SetUpTestCase() {
}

void test_set_thread_fixture::TearDownTestCase() {
}

void test_set_thread_fixture::SetUp() {
}

void test_set_thread_fixture::TearDown() {
}

TEST_F( test_set_thread_fixture, TestOpOtherIdAny ) {

	std::string msg;

	std::exception_ptr p = nullptr;

	in.str( "+$OK#9a" );

	client client( in, os );

	try {
		client.setThread( g_gdbrsp_constants.SET_THREAD_OP_OTHER, g_gdbrsp_constants.THREAD_ID_ANY );
	} catch( ... ) {
		p = std::current_exception();
	}

	if ( nullptr != p ) {
		try {
			std::rethrow_exception( p );
		} catch( std::exception & e ) {
			msg = e.what();
		}
	}

	EXPECT_TRUE( nullptr == p ) << "exception was thrown with message " << msg;
}

TEST_F( test_set_thread_fixture, TestOpStepIdAny ) {

	std::string msg;

	std::exception_ptr p = nullptr;

	in.str( "+$OK#9a" );

	client client( in, os );

	try {
		client.setThread( g_gdbrsp_constants.SET_THREAD_OP_STEP_AND_CONTINUE, g_gdbrsp_constants.THREAD_ID_ANY );
	} catch( ... ) {
		p = std::current_exception();
	}

	if ( nullptr != p ) {
		try {
			std::rethrow_exception( p );
		} catch( std::exception & e ) {
			msg = e.what();
		}
	}

	EXPECT_TRUE( nullptr == p ) << "exception was thrown with message " << msg;
}

TEST_F( test_set_thread_fixture, TestOpOtherIdAll ) {

	std::string msg;

	std::exception_ptr p = nullptr;

	in.str( "+$OK#9a" );

	client client( in, os );

	try {
		client.setThread( g_gdbrsp_constants.SET_THREAD_OP_OTHER, g_gdbrsp_constants.THREAD_ID_ALL );
	} catch( ... ) {
		p = std::current_exception();
	}

	if ( nullptr != p ) {
		try {
			std::rethrow_exception( p );
		} catch( std::exception & e ) {
			msg = e.what();
		}
	}

	EXPECT_TRUE( nullptr == p ) << "exception was thrown with message " << msg;
}

TEST_F( test_set_thread_fixture, TestOpStepIdAll ) {

	std::string msg;

	std::exception_ptr p = nullptr;

	in.str( "+$OK#9a" );

	client client( in, os );

	try {
		client.setThread( g_gdbrsp_constants.SET_THREAD_OP_STEP_AND_CONTINUE, g_gdbrsp_constants.THREAD_ID_ALL );
	} catch( ... ) {
		p = std::current_exception();
	}

	if ( nullptr != p ) {
		try {
			std::rethrow_exception( p );
		} catch( std::exception & e ) {
			msg = e.what();
		}
	}

	EXPECT_TRUE( nullptr == p ) << "exception was thrown with message " << msg;
}

TEST_F( test_set_thread_fixture, TestOpInvalidIdAny ) {

	std::string msg;

	std::exception_ptr p = nullptr;

	in.str( "+$OK#9a" );

	client client( in, os );

	try {
		client.setThread( 'q', g_gdbrsp_constants.THREAD_ID_ANY );
	} catch( ... ) {
		p = std::current_exception();
	}

	if ( nullptr != p ) {
		try {
			std::rethrow_exception( p );
		} catch( std::exception & e ) {
			msg = e.what();
		}
	}

	EXPECT_TRUE( nullptr == p ) << "exception was thrown with message " << msg;
}

TEST_F( test_set_thread_fixture, TestOpOtherIdInvalid ) {

	std::string msg;

	std::exception_ptr p = nullptr;

	in.str( "+$OK#9a" );

	client client( in, os );

	try {
		client.setThread( g_gdbrsp_constants.SET_THREAD_OP_OTHER, -2 );
	} catch( ... ) {
		p = std::current_exception();
	}

	EXPECT_TRUE( nullptr != p ) << "exception was not thrown!";
}

TEST_F( test_set_thread_fixture, TestOpInvalidIdInvalid ) {

	std::string msg;

	std::exception_ptr p = nullptr;

	in.str( "+$OK#9a" );

	client client( in, os );

	try {
		client.setThread( 'q', -2 );
	} catch( ... ) {
		p = std::current_exception();
	}

	EXPECT_TRUE( nullptr != p ) << "exception was not thrown!";
}

} /* namespace rsp */
} /* namespace gdb */
} /* namespace org */
